@@ -134,8 +134,9 @@ namespace QuantLib {
 
     void Gaussian1dNonstandardSwaptionEngine::calculate() const {
 
-        QL_REQUIRE(arguments_.settlementType == Settlement::Physical,
-                   "cash-settled swaptions not yet implemented ...");
+        QL_REQUIRE(arguments_.settlementMethod != Settlement::ParYieldCurve,
+                   "cash settled (ParYieldCurve) swaptions not priced with "
+                   "Gaussian1dNonstandardSwaptionEngine");
 
         Date settlement = model_->termStructure()->referenceDate();
 
