@@ -65,6 +65,8 @@ namespace QuantLib {
             || (y == 2015 && d <= 3 && m == January)
             || (y == 2017 && d == 2 && m == January)
             || (y == 2018 && d == 1 && m == January)
+            || (y == 2018 && d == 31 && m == December)
+            || (y == 2019 && d == 1 && m == January)
             // Chinese New Year
             || (y == 2004 && d >= 19 && d <= 28 && m == January)
             || (y == 2005 && d >=  7 && d <= 15 && m == February)
@@ -84,6 +86,7 @@ namespace QuantLib {
             || (y == 2017 && ((d >= 27 && m == January) ||
                               (d <= 2 && m == February)))
             || (y == 2018 && (d >= 15 && d <= 21 && m == February))
+            || (y == 2019 && d >= 4 && d <= 8 && m == February)
             // Ching Ming Festival
             || (y <= 2008 && d == 4 && m == April)
             || (y == 2009 && d == 6 && m == April)
@@ -96,6 +99,7 @@ namespace QuantLib {
             || (y == 2016 && d == 4 && m == April)
             || (y == 2017 && d >= 3 && d <= 4 && m == April)
             || (y == 2018 && d >= 5 && d <= 6 && m == April)
+            || (y == 2019 && d == 5 && m == April)
             // Labor Day
             || (y <= 2007 && d >= 1 && d <= 7 && m == May)
             || (y == 2008 && d >= 1 && d <= 2 && m == May)
@@ -111,6 +115,7 @@ namespace QuantLib {
             || (y == 2016 && d >= 1 && d <=2 && m == May)
             || (y == 2017 && d == 1 && m == May)
             || (y == 2018 && ((d == 30 && m == April) || (d == 1 && m == May)))
+            || (y == 2019 && d == 1 && m == May)
             // Tuen Ng Festival
             || (y <= 2008 && d == 9 && m == June)
             || (y == 2009 && (d == 28 || d == 29) && m == May)
@@ -123,6 +128,7 @@ namespace QuantLib {
             || (y == 2016 && d >= 9 && d <= 10 && m == June)
             || (y == 2017 && d >= 29 && d <= 30 && m == May)
             || (y == 2018 && d == 18 && m == June)
+            || (y == 2019 && d == 7 && m == June)
             // Mid-Autumn Festival
             || (y <= 2008 && d == 15 && m == September)
             || (y == 2010 && d >= 22 && d <= 24 && m == September)
@@ -133,8 +139,9 @@ namespace QuantLib {
             || (y == 2015 && d == 27 && m == September)
             || (y == 2016 && d >= 15 && d <= 16 && m == September)
             || (y == 2018 && d == 24 && m == September)
+            || (y == 2019 && d == 13 && m == September)
             // National Day
-            || (y <= 2007 && d >= 1 && d <= 7 && m == October)
+            || (y <= 2007 && d >= 1 && d <= 7 && m == October) 
             || (y == 2008 && ((d >= 29 && m == September) ||
                               (d <= 3 && m == October)))
             || (y == 2009 && d >= 1 && d <= 8 && m == October)
@@ -147,6 +154,7 @@ namespace QuantLib {
             || (y == 2016 && d >= 3 && d <= 7 && m == October)
             || (y == 2017 && d >= 2 && d <= 6 && m == October)
             || (y == 2018 && d >= 1 && d <= 5 && m == October)
+            || (y == 2019 && ((d == 30 && m == September) || (d >= 1 && d <= 4 && m == October)))
             // 70th anniversary of the victory of anti-Japaneses war
             || (y == 2015 && d >= 3 && d <= 4 && m == September)
             )
@@ -263,7 +271,13 @@ namespace QuantLib {
             Date(8, April, 2018),
             Date(28, April, 2018),
             Date(29, September, 2018),
-            Date(30, September, 2018)
+            Date(30, September, 2018),
+            Date(29, December, 2018),
+            // 2019
+            Date(2, February, 2019),
+            Date(3, February, 2019),
+            Date(29, September, 2019),
+            Date(12, October, 2019)
         };
         static const Size n =
             sizeof(working_weekends)/sizeof(working_weekends[0]);
